#define F_CPU         16000000

#include "Resources.h"
#include "GLI.h"
#include "snake.h"

#include <avr/interrupt.h>
#include <avr/io.h>
#include <util/delay.h>

#include <stdio.h>
#include <stdlib.h>
#include <string.h>



ISR(PCINT1_vect)
{
	if ((PINB & (1 << PB5)) == 0 && direction != UP) {	
		direction = DOWN;
		toggle = 1;
		
	} else if ((PINB & (1 << PB4)) == 0 && direction != RIGHT) {	
		direction = LEFT;
		toggle = 1;	
		
	} else if ((PINB & (1 << PB7)) == 0 && direction != DOWN) {	
		toggle = 1;
		direction = UP;
		
	} else if ((PINB & (1 << PB6)) == 0 && direction != LEFT) {	
		toggle = 1;
		direction = RIGHT;
		
	} 	
}

ISR(TIMER1_COMPA_vect) {
   if(food) {
		PORTB |= (1 << PB2);
		food = 0;
	} else {
		PORTB &= ~(1 << PB2);
	}
}

ISR(TIMER2_COMPA_vect)
{
	if (toggle) {
		PORTB ^= (1 << PB0); 
		toggle = 0;	
	} else {
		PORTB &= ~(1 << PB0); 	
	}
}

void draw_text(char msg[], unsigned int y) {
    GLI_DrawString((SCREEN_WIDTH - g_SmallFont.CharWidth * strlen(msg)) / 2 - 1, y, &g_SmallFont, msg);
}

void init_buttons() {
	/* Setez intrarea */
	DDRB &= ~(1 << PB7);
	DDRB &= ~(1 << PB6);
	DDRB &= ~(1 << PB5);
	DDRB &= ~(1 << PB4);

	/* Activez rezistenta de pull-up */
	PORTB |= (1 << PB7);
	PORTB |= (1 << PB6);
	PORTB |= (1 << PB5);
	PORTB |= (1 << PB4);
	
	PCICR  = (1 << PCIE1);
    PCMSK1 = (1 << PCINT14) | (1 << PCINT13) | (1 << PCINT12) | (1 << PCINT15);	
}

void init_led() {
	/* Setez iesirea */
	DDRB |= (1 << PB1);
	DDRB |= (1 << PB2);
	DDRB |= (1 << PB3);

	/* Sting LED-ul */
	PORTB &= ~(1 << PB1);
	PORTB &= ~(1 << PB2);
	PORTB &= ~(1 << PB3);
}

void init_buzzer() {
	DDRB |= (1 << PB0);
	PORTB &= ~(1 << PB0);
		
	TCCR2B |= (1 << WGM12); //CTC mode
	TIMSK2 |= (1 << OCIE2A);
	OCR1A   = 255;
	TCCR2B |= ((1 << CS10) | (1 << CS12));
}

void init() {
	// initializez LCD
	GLI_Initialize();
    
	init_buttons();
	init_led();
	init_buzzer();
	put_food();
	
	TCCR1B |= (1 << WGM12); //CTC mode
	TIMSK1 |= (1 << OCIE1A);
	OCR1A   = 2000;
	TCCR1B |= ((1 << CS10) | (1 << CS12));
	
	dim_x[0] = 9;
	dim_y[0] = 9;
	
	dim_x[1] = 12;
	dim_y[1] = 9;
	
	dim_x[2] = 15;
	dim_y[2] = 9;
	
	dim_x[3] = 18;
	dim_y[3] = 9;
	
	length = 4;
}

void move_dim() {
	int i;
	
	switch(direction) {
	case RIGHT:
		for (i = 0; i < length - 1; i++) {
			dim_x[i] = dim_x[i + 1];
			dim_y[i] = dim_y[i + 1];
		}
		
		dim_x[length - 1] += FACTOR;
		break;
	
	case DOWN:
		for (i = 0; i < length - 1; i++) {
			dim_x[i] = dim_x[i + 1];
			dim_y[i] = dim_y[i + 1];
		}
		
		dim_y[length - 1] += FACTOR;
		break;
		
	case LEFT:
		for (i = 0; i < length - 1; i++) {
			dim_x[i] = dim_x[i + 1];
			dim_y[i] = dim_y[i + 1];
		}
		
		dim_x[length - 1] -= FACTOR;
		break;
		
	case UP:
		for (i = 0; i < length - 1; i++) {
			dim_x[i] = dim_x[i + 1];
			dim_y[i] = dim_y[i + 1];
		}
		
		dim_y[length - 1] -= FACTOR;
		break;	
		
	default: break;
	}
}

void draw_snake() {
	int i;
		
	for (i = 0; i < length; i++) {
		GLI_DrawBitmap(dim_x[i], dim_y[i], &g_SnakeBig);
	}
	GLI_DrawBitmap(x_food, y_food, &g_SnakeBig);
}

void draw_border() {
	int i;
	for (i = 0; i < SCREEN_WIDTH; i += 2) {
		GLI_DrawBitmap(i, 0, &g_Border);
		GLI_DrawBitmap(i, 62, &g_Border);
	}
	
	for (i = 0; i < SCREEN_HEIGHT; i += 2) {
		GLI_DrawBitmap(0, i, &g_Border);
		GLI_DrawBitmap(126, i, &g_Border);
	}
}

void put_food() {
	int i, valid = 1;
	
	
	while (valid) {
		x_food = rand() % 126;
		y_food = rand() % 62;
		
		if (x_food % FACTOR == 0 && y_food % FACTOR == 0 && x_food >= 2 && y_food >= 2) {
			valid = 0;
			for (i = 0; i < length; i++) {
				if(x_food == dim_x[i] || y_food == dim_y[i]) {
					valid = 1;
					break;
				}
			}
		}
	}
}

void get_food() {	
	
	if(dim_x[length-1] == x_food && dim_y[length-1] == y_food){
		food = 1;
		put_food();
		length++;
		
		switch(direction) {
		case RIGHT:
			dim_x[length-1] = dim_x[length-2] + FACTOR;
			dim_y[length-1] = dim_y[length-2];
			break;
		case DOWN:
			dim_x[length-1] = dim_x[length-2];
			dim_y[length-1] = dim_y[length-2] + FACTOR;
			break;
		case LEFT:
			dim_x[length-1] = dim_x[length-2] - FACTOR;
			dim_y[length-1] = dim_y[length-2];
			break;
		case UP:
			dim_x[length-1] = dim_x[length-2];
			dim_y[length-1] = dim_y[length-2] - FACTOR;
			break;
		default: break;
		}
		
		delay -= 3;
	}
}

void move_snake() {
	GLI_Clear(GLI_COLOR_BLACK);
	draw_border();
	draw_snake();
	GLI_Present();	
}

int detect_collision(){
	int i;
	
	for(i = 0; i < length - 1; i++){
		if(dim_x[i] == dim_x[length - 1] && dim_y[i] == dim_y[length - 1])
			return 0;
	}
	
	return 1;
}

int main() {
	int i; 
	
	init();	
	sei();
	
	GLI_Clear(GLI_COLOR_BLACK);
	
	while(dim_x[length-1] < 126 && dim_y[length-1] < 62 && dim_x[length-1] > -1 && dim_y[length-1] > -1) {
		if(detect_collision() == 0)
			break;
		move_snake();
		get_food();
		move_dim();
		
		_delay_ms(delay);
	}
	
	for (i = 0; i < 3; i++) {
		PORTB |= (1 << PB1);		
		_delay_ms(500);
		PORTB &= ~(1 << PB1);
		_delay_ms(500);
	}
	
	return 0;
}
